@@ -31,6 +31,9 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <sys/utsname.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
 #include <errno.h>
 
@@ -41,7 +44,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnome/libgnometypebuiltins.h>
-
+#include <gdk/gdkx.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -66,6 +69,7 @@
 #define USE_PROXY_AUTH	"/system/http_proxy/use_authentication"
 #define PROXY_USER	"/system/http_proxy/authentication_user"
 #define PROXY_PASSWORD	"/system/http_proxy/authentication_password"
+#define ACCESSIBILITY_KEY "/desktop/gnome/interface/accessibility"
 
 static GOptionData gopt_data;
 static int bug_count = 0;
@@ -89,6 +93,8 @@ enum {
 	NETWORK_UNKNOWN
 };
 
+static void fill_stderr_info (GladeXML *xml);
+
 
 static void
 buddy_error (GtkWidget *parent, const char *msg, ...)
@@ -243,10 +249,10 @@ network_error (SoupMessage *msg, GladeXML *xml)
 						      GTK_DIALOG_MODAL,
 						      GTK_MESSAGE_QUESTION,
 						      GTK_BUTTONS_YES_NO,
-						      "<span weight=\"bold\">Network Connection Error</span>\n"
+						      _("<span weight=\"bold\">Network Connection Error</span>\n"
 						      "Maybe no Network Connection available.\n"
 						      "Do you want to store this report until "
-						      "a Network Connection is available?");
+						      "a Network Connection is available?"));
 	res = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
@@ -407,7 +413,7 @@ bug_sent (SoupMessage *msg, GladeXML *xml)
 					text = g_strdup_printf (_("The email address you provided is not valid."));
 				} else if (g_str_equal (ptr, "account_disabled")) {
 					text = g_strdup_printf (_("The account associated with the email address "
-				                                  "provided, has been disabled."));
+				                                  "provided has been disabled."));
 				} else if (g_str_equal (ptr, "product_doesnt_exist")) {
 					text = g_strdup_printf (_("The product specified doesn't exist or has been "
 				                                  "renamed.  Please upgrade to the latest version."));
@@ -483,8 +489,8 @@ set_proxy (SoupSession *session)
 		port = 80;
 
 	if (gconf_client_get_bool (gconf_client, USE_PROXY_AUTH, NULL)) {
-		username = gconf_client_get_string (gconf_client, PROXY_HOST_KEY, NULL);
-		password = gconf_client_get_string (gconf_client, PROXY_HOST_KEY, NULL);
+		username = gconf_client_get_string (gconf_client, PROXY_USER, NULL);
+		password = gconf_client_get_string (gconf_client, PROXY_PASSWORD, NULL);
 	}
 
 	if (username && password)
@@ -623,7 +629,7 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 					bug_count++;
 					soup_xmlrpc_message_persist (msg);
         				soup_session_queue_message (session, SOUP_MESSAGE (msg),
-               		                    			    previous_sent, xml);
+               		                    			    (SoupMessageCallbackFn)previous_sent, xml);
 					g_free (contents);
 				}
 				g_free (path);
@@ -777,33 +783,48 @@ show_pending_checkbox_if_pending (GladeXML *xml)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
 		gtk_widget_show (check);
 	}
+	g_free (dirname);
 }
 
 static void
 known_app_finished (GladeXML *xml)
 {
 	GtkWidget *email_entry;
+	GtkWidget *button;
 	char *default_email;
+	char *lang_note, *label_text;
+	const char *en_lang_note = N_("\nPlease write your report in English, if possible.\n");
+
+	fill_stderr_info (xml);
 	
 	gtk_widget_show (glade_xml_get_widget (xml, "final-box"));
 	gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
 	gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
 
+	lang_note = gettext (en_lang_note);
+
+	label_text = g_strconcat (_("The information about the crash has been successfully collected. "
+				    "Please provide some background about what you were doing when "
+				    "the application crashed.\n\n"
+
+				    "A valid email address is required.  This will allow developers to "
+				    "contact you for more information if necessary.\n\n"
+				    "Sensitive data may be present in the debug information, so please "
+				    "review details below if you are concerned about transmitting "
+				    "passwords or other sensitive data.\n"),
+				  strcmp (lang_note, en_lang_note) ? lang_note : NULL,
+				  NULL);
 
 	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")),
-			    _("The information about the crash has been successfully collected. "
-			      "Please provide some background about what you were doing when "
-			      "the application crashed.\n\n"
-			      "A valid email address is required.  This will allow developers to "
-			      "contact you for more information if necessary.\n\n" 
-			      "Sensitive data may be present in the debug information, so please "
-			      "review details below if you are concerned about transmitting "
-	                      "passwords or other sensitive data.\n"));
+			    label_text);
+
+	g_free (label_text);
 
 	show_pending_checkbox_if_pending (xml);
 	
-	g_signal_connect (glade_xml_get_widget (xml, "send-button"), "clicked", 
+	button = glade_xml_get_widget (xml, "send-button");
+	g_signal_connect (button, "clicked", 
 	                  G_CALLBACK (on_send_clicked), xml);
 
 	email_entry = glade_xml_get_widget (xml, "email-entry");
@@ -814,6 +835,8 @@ known_app_finished (GladeXML *xml)
 	if (default_email != NULL) {
 		gtk_entry_set_text (GTK_ENTRY (email_entry), default_email);
 		g_free (default_email);
+	} else {
+		gtk_widget_set_sensitive (button, FALSE);
 	}
 
 	gtk_widget_grab_focus (glade_xml_get_widget (xml, "details-view"));
@@ -894,6 +917,8 @@ unknown_app_finished (GladeXML *xml)
 {
 	GtkWidget *button;
 
+	fill_stderr_info (xml);
+	
 	/* don't need user input, so hide these widgets */
 	gtk_widget_hide (glade_xml_get_widget (xml, "final-box"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "progressbar"));
@@ -1054,6 +1079,134 @@ fill_include_file (char *filename, GladeXML *xml)
 }
 
 
+static void
+fill_system_info (GladeXML *xml)
+{
+	GConfClient *gconf_client;
+	GtkTextView *text_view;
+	GtkTextIter end;
+	GtkTextBuffer *buffer;
+	GString *system_info;
+	struct utsname uts_buf;
+	char *str;
+	gboolean has_selinux, enforcing, a11y;
+	
+	g_return_if_fail (xml != NULL);
+	
+	system_info = g_string_new ("");
+
+	if (uname (&uts_buf) == 0) {
+		g_string_append_printf (system_info, "System: %s %s %s %s\n", uts_buf.sysname, uts_buf.release, uts_buf.version, uts_buf.machine);
+	}
+
+	/* X server checks */
+	g_string_append_printf (system_info, "X Vendor: %s\n", ServerVendor (gdk_display));
+	g_string_append_printf (system_info, "X Vendor Release: %d\n", VendorRelease (gdk_display));
+	
+	
+	/* Selinux checks */
+	has_selinux = FALSE;
+	if (g_file_get_contents ("/proc/filesystems", &str, NULL, NULL)) {
+		has_selinux = strstr (str, "selinuxfs") != NULL;
+		g_free (str);
+	}
+	if (has_selinux) {
+		enforcing = TRUE;
+		if (g_file_get_contents ("/selinux/enforce", &str, NULL, NULL)) {
+			enforcing = strcmp (str, "0") != 0;
+			g_free (str);
+		}
+		g_string_append_printf (system_info, "Selinux: %s\n", enforcing?"Enforcing":"Permissive");
+	} else {
+		g_string_append_printf (system_info, "Selinux: No\n");
+	}
+
+	gconf_client = gconf_client_get_default ();
+	a11y = gconf_client_get_bool (gconf_client, ACCESSIBILITY_KEY, NULL);
+	g_object_unref (gconf_client);
+	g_string_append_printf (system_info, "Accessibility: %s\n", a11y?"Enabled":"Disabled");
+
+	g_string_append (system_info, "\n");
+	
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+	gtk_text_buffer_insert (buffer, &end, system_info->str, system_info->len);
+
+	g_string_free (system_info, TRUE);
+}
+
+static void
+fill_stderr_info (GladeXML *xml)
+{
+	GtkTextView *text_view;
+	GtkTextIter end;
+	GtkTextBuffer *buffer;
+	GString *stderr_info;
+	char *str, *file;
+	gchar **lines;
+	int n_lines, i;
+	
+	g_return_if_fail (xml != NULL);
+	
+	stderr_info = g_string_new ("");
+
+	/* .xsession-errors: read file */
+	file = g_build_filename (g_get_home_dir (), ".xsession-errors", NULL);
+	if (g_file_get_contents (file, &str, NULL, NULL)) {
+		lines = g_strsplit (str, "\n", -1);
+		g_free (str);
+		n_lines = 0;
+		while (lines[n_lines] != NULL) {
+			n_lines++;
+		}
+
+		if (n_lines > 0) {
+			struct stat buf;
+			char *mtime_age = NULL;
+			time_t age = 0; 
+
+			
+			if (stat (file, &buf) == 0) {
+				age = time (NULL) - buf.st_mtime;
+				if (age > 5) {
+					mtime_age = g_strdup_printf (" (%d sec old)", age);
+				}
+			}
+				
+			g_string_append_printf (stderr_info,
+						"\n\n----------- .xsession-errors%s ---------------------\n",
+						mtime_age?mtime_age:"");
+			g_free (mtime_age);
+		}
+			
+		for (i = MAX (0, n_lines-16); i < n_lines; i++) {
+			if (lines[i][0] != 0) {
+				/* Limit line length to 200 chars to avoid excessive data */
+				if (strlen (lines[i]) > 200) {
+					lines[i][200] = 0;
+				}
+				
+				g_string_append_printf (stderr_info, "%s\n", lines[i]);
+			}
+		}
+		if (n_lines > 0)
+			g_string_append (stderr_info, "--------------------------------------------------\n");
+		
+		g_strfreev (lines);
+	}
+	g_free (file);
+
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+	gtk_text_buffer_insert (buffer, &end, stderr_info->str, stderr_info->len);
+	
+	g_string_free (stderr_info, TRUE);
+}
+
+
+
 
 int
 main (int argc, char *argv[])
@@ -1138,7 +1291,7 @@ main (int argc, char *argv[])
 	/* get some information about the gnome version */
 	gnome_version = get_gnome_version_info ();
 	if (gnome_version == NULL) {
-		buddy_error (NULL, _("Bug Buddy was unable to retreive information regarding "
+		buddy_error (NULL, _("Bug Buddy was unable to retrieve information regarding "
 		                     "the version of GNOME you are running.  This is most likely "
 		                     "due to a missing installation of gnome-desktop.\n"));
 		g_object_unref (program);
@@ -1174,6 +1327,8 @@ main (int argc, char *argv[])
 		gtk_statusbar_push (GTK_STATUSBAR (statusbar),
 			    	    DEBUGGING, _("Collecting info from the crash..."));
 
+		fill_system_info (xml);
+		
 		fill_proccess_info (gopt_data.pid, xml);
 
 		if (gopt_data.pid > 0) {
@@ -1209,8 +1364,10 @@ main (int argc, char *argv[])
 		/* No binary file, so this is a non-crashing bug. Look the application from the --package arg */
 		GtkWidget *email_entry;
 		char *default_email;
+ 		char *msg, *lang_note;
+                const char *en_lang_note = N_("\nPlease write your report in English, if possible.\n");
 		
-		app = g_hash_table_find (apps, bugzilla_search_for_package, gopt_data.package);
+		app = g_hash_table_find (apps, (GHRFunc)bugzilla_search_for_package, gopt_data.package);
 		if (app == NULL) {
 			/* Fallback to binary name */
 			app = g_hash_table_lookup (apps, gopt_data.package);
@@ -1220,11 +1377,19 @@ main (int argc, char *argv[])
 		                     	     gopt_data.package);
 			return 0;
 		}
-		s = g_markup_printf_escaped (_("Thank you for helping us improving our software.\n"
-				               "Please fill your suggestions/error information for %s application.\n"),
-                                              	app->name);
+
+		lang_note = gettext (en_lang_note);
+		msg = g_strconcat (_("Thank you for helping us improving our software.\n"
+				     "Please fill your suggestions/error information for %s application.\n"),
+				   strcmp (lang_note, en_lang_note)? lang_note: NULL, 
+				   NULL);
+
+		s = g_markup_printf_escaped (msg, app->name);
+
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
 		g_free (s);
+
+                g_free (msg);
 		
 		g_object_set_data (G_OBJECT (xml), "app", app);
 	
@@ -1276,6 +1441,8 @@ main (int argc, char *argv[])
 	
 	gtk_main ();
 
+	g_hash_table_destroy (apps);
+
 	g_object_unref (program);
 	return 0;
 }
